from Atom import Atom
# Deal with the radii
import Radii
R = Radii.Radii()

# This class has been implimented so that the _set_atom_polarity() and _set_radius()
# methods can be altered easily. At the moment _set_atom_polarity() simply defines
# all atoms with atomType 'O', 'N' and 'H2O' as polar. However, C(C)NC, for
# example, is not polar.
# Furthermore currently each atom uses the average radii used in the
# original PRO_ACT. More recent radii have been determined by Li & Nussinov
# with specific atom types separating NH2 and NH3+ as separate atom types.
# To impliment a scheme based on the atom types used by Li & Nussinov would
# require two more LigandAtom attributes 'bondedAtoms' and 'bondedAtomBonds'
# which could be used to determine the specific atom types.

LigandAtomError = "ligand atom error"

class LigandAtom(Atom):
    """Class LigandAtom

    The LigandAtom class inherits all the attributes and methods of the Atom
    class. Currently it has no extra attributes but this could change if the
    atomic radii were to be defined on a basis other than the atom type.

    Examples:
    >>> from LigandAtom import LigandAtom
    >>> import Radii
    >>> R = Radii.Radii()
    >>> R.radii_set = 'LI_AND_NUSSINOV'
    >>> la = LigandAtom(1, 'O', 'O', 0.0, 0.0, 0.0, 2, 0)
    >>> la.atom_type
    'O'
    >>> la.atom_polarity
    0
    >>> '%.2f' % la.char_apol_radius
    '1.51'
    >>> '%.2f' % la.char_pol_radius
    '1.38'
    >>> '%.2f' % la.char_pol_radius_max
    '1.50'
    >>> '%.2f' % la.close_radius
    '2.75'
    >>> '%.2f' % la.far_radius
    '3.05'
    >>> '%.2f' % la.probe_radius
    '1.77'

    """

    def __init__(self, atom_org_id, atom_org_name, atom_type, x, y, z,
                 sub_atom_type, molecule_instance):
        """
        Initialise the object.

        atom_org_id: int
        atom_org_name: string
        atom_type: string
        x: float
        y: float
        z: float
        sub_atom_type: int
        molecule_instance: int

        This method sets the following attributes of the atom:
        1) _atom_org_id
        2) _atom_org_name
        3) _atom_type
        4) _x, _y, _z
        5) _sub_atom_type
        6) _molecule_instance
        7) _atom_polarity
        8) various radii...

        """

        Atom.__init__(self, atom_org_id, atom_org_name, atom_type, x, y, z,
                      sub_atom_type, molecule_instance)
        self._set_atom_polarity()
        self._set_radius()
        

    # Attributes:
    
    # Operations
    def _set_radius(self):
        """function _set_radius

        This method sets the various atomic radii defined in the Atom class.
        
        """
        if self.atom_polarity == None:
            raise LigandAtomError, "_set_radius() requires polarity to be set"
        radii = R.radii[self.atom_type.upper()]
        self.char_apol_radius = radii['charApolRadius']
        if self.atom_polarity == 0:
            self.char_pol_radius = radii['charPolRadius']
            self.char_pol_radius_max = radii['charPolRadiusMax']
        self.close_radius = radii['closeRadius']
        self.far_radius = radii['farRadius']
        self.probe_radius = radii['probeRadius']
    
    def _set_atom_polarity(self):
        """function _set_atom_polarity
        
        At the moment this function simply says that all atoms with atomType
        'O', 'N' and 'H2O' are polar. This can be improved later on. For
        example CN(C)C should really be apolar.

        """
        if (self.atom_type == 'O'
            or self.atom_type == 'N'
            or self.atom_type == 'H2O'):
            self.atom_polarity = 0  # polar
        else:
            self.atom_polarity = 2  # apolar
    
    def in_contact(self, target):
        """
        This method checks to see if the Atom is in contact with the target
        Atom. The function returns an integer based on the type of contact.
        The types of contacts are defined as:
        1) No contact     - (-1)
        2) Polar contact  - (1)
        3) Apolar contact - (2)

        target: Atom

        Atoms of the same residue/ligand are never in contact. Atoms part of
        the same peptide bond are never in contact.

        returns int
        """
        # Check to see if target atom is also a ligand atom
        if isinstance(target, LigandAtom):
            return -1
        return Atom.in_contact(self, target)

# Test LigandAtom class by the command 'python LigandAtom.py'
if __name__ == "__main__":
    import doctest
    doctest.testmod()
    import unittest
    suite = doctest.DocFileSuite('doc_tests/test_LigandAtom.txt',
        optionflags=doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE)
    unittest.TextTestRunner().run(suite)
