class Mol2Error(Exception):
    """
    Custom Mol2 error class.
    """
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class Mol2:
    """Class for interpreting molecules in mol2 file format."""
    
    def __init__(self, ligand_file):
        """
        Initialise the molecule.
        """
        records = self._get_records(ligand_file)
        self.molecule_rec = self._load_molecule_rec(records['MOLECULE'])
        self.atom_rec = self._load_atom_rec(records['ATOM'])
        self.bond_rec = self._load_bond_rec(records['BOND'])

    def _load_molecule_rec(self, record_data):
        """Returns the molecule record information in a dictionnary format."""
        
        # define molecule dictionnary
        mol = {}
        
        # test for six data lines
        if len(record_data) < 6:
            raise Mol2Error('molecule record has less than six lines')
        elif len(record_data) > 6:
            raise Mol2Error('molecule record has more than six lines')
            
        # molecule name
        if record_data[0] == '':
            raise Mol2Error('molecule record has no name')
        else:
            mol['mol_name'] = record_data[0]
        
        # num_atoms, num_bonds, num_subst, num_feat, num_sets
        if record_data[1]== '':
            raise Mol2Error('molecule record has no number of atoms')
        else:
            tmp = record_data[1].split()
            
            # num_atoms
            try:
                tmp[0]                            # check existance
            except IndexError:                    # or raise error
                if STRICT:
                    raise Mol2Error('molecule record has no atoms')
            else:                                 # if ok set it
                mol['num_atoms'] = int(tmp[0])
            
            # num_bonds
            try:
                tmp[1]                            # check existance
            except IndexError:                    # or raise error
                mol['num_bonds'] = ''
            else:                                 # if ok set it
                mol['num_bonds'] = int(tmp[1])
            
            # num_subst
            try:
                tmp[2]                            # check existance
            except IndexError:                    # or raise error
                mol['num_subst'] = ''
            else:                                 # if ok set it
                mol['num_subst'] = int(tmp[2])
            
            # num_feat
            try:
                tmp[3]                            # check existance
            except IndexError:                    # or raise error
                mol['num_feat'] = ''
            else:                                 # if ok set it
                mol['num_feat'] = int(tmp[3])
            
            # num_sets
            try:
                tmp[4]                            # check existance
            except IndexError:                    # or raise error
                mol['num_sets'] = ''
            else:                                 # if ok set it
                mol['num_sets'] = int(tmp[4])
        
        # molecule type
        if record_data[2] == '':
            raise Mol2Error('molecule record has no molecule type')
        else:
            mol['mol_type'] = record_data[2]
        
        # charge type
        if record_data[3] == '':
            raise Mol2Error('molecule record has no charge type')
        else:
            mol['charge_type'] = record_data[3]
        
        # status bits
        if record_data[4] == '':
            mol['status_bits'] = ''
        else:
            mol['status_bits'] = record_data[4]
        
        # molecule comment
        if record_data[5] == '':
            mol['mol_comment'] = ''
        else:
            if record_data[4] == '':
                raise Mol2Error('molecule record comment without status bit line')
            else:
                mol['mol_comment'] = record_data[5]
        
        return mol
    
    def _load_atom_rec(self, record_data):
        """
        Return a  of list the atom record information in a dictionnary format.
        """
        
        # define atom list
        atoms = []
        
        # create dictonnary format
        for line in record_data:
        
            # skip blank lines
            line_no_whitespaces = line.strip()
            if line_no_whitespaces == '': continue
            
            # create temporary list of data
            tmp = line.split()
            
            # define atom dictionnary
            dic = {}
            
            # atom id
            try:
                tmp[0]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Error('atom record has no atom id')
            else:                                 # if ok set it
                dic['atom_id'] = int(tmp[0])
            
            # atom name
            try:
                tmp[1]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Error('atom record has no atom name')
            else:                                 # if ok
                dic['atom_name'] = tmp[1]         # set it
            
            # x coordinate
            try:
                tmp[2]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Error('atom record has no x coordinate')
            else:                                 # if ok
                dic['x'] = float(tmp[2])    # set it
            
            # y coordinate
            try:
                tmp[3]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Error('atom record has no y coordinate')
            else:                                 # if ok
                dic['y'] = float(tmp[3])    # set it
            
            # z coordinate
            try:
                tmp[4]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Error('atom record has no z coordinate')
            else:                                 # if ok
                dic['z'] = float(tmp[4])    # set it
            
            # atom type
            try:
                tmp[5]                            # check existance
            except IndexError:
                dic['atom_type'] = ''
            else:                                 # if it exists
                dic['atom_type'] = tmp[5]         # set it
            
            # substructure id
            try:
                tmp[6]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Error('atom record has no atom type')
            else:                                 # if it exists set it
                dic['subst_id'] = int(tmp[6])
            
            # substructure name
            try:
                tmp[7]                            # check existance
            except IndexError:
                dic['subst_name'] = ''
            else:                                 # if it exists
                dic['subst_name'] = tmp[7]        # set it
            
            # charge
            try:
                tmp[8]                            # check existance
            except IndexError:
                dic['charge'] = ''
            else:                                 # if it exists set it
                dic['charge'] = float(tmp[8])
            
            # status bits
            try:
                tmp[9]                            # check existance
            except IndexError:
                dic['status_bit'] = ''
            else:                                 # if it exists
                dic['status_bit'] = tmp[9]       # set it
            
            # appned atom dictionnary to atom list   
            atoms.append(dic)
        
        return atoms
    
    def _load_bond_rec(self, record_data):
        """Return a list of the bond information in a dictionnary format."""
        
        bonds = []
        
        # create dictionnary format
        for line in record_data:
        
            # skip blank lines
            line_no_whitespaces = line.strip()
            if line_no_whitespaces == '': continue
            
            # create temporary list of data
            tmp = line.split()

            # define bond dictionnary
            dic = {}
            
            # bond id
            try:
                tmp[0]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Error('bond record has no bond id')
            else:                                 # if ok set it
                dic['bond_id'] = int(tmp[0])
            
            # original atom id
            try:
                tmp[1]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Error('bond record has no original atom id')
            else:                                 # if ok set it
                dic['origin_atom_id'] = int(tmp[1])
            
            # target atom id
            try:
                tmp[2]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Error('bond record has no target atom id')
            else:                                 # if ok set it
                dic['target_atom_id'] = int(tmp[2])
            
            # bond type
            try:
                tmp[3]                            # check existance
            except IndexError:                    # or raise error
                raise Mol2Errora('bond record has no bond type')
            else:                                 # if ok
                dic['bond_type'] = tmp[3]         # set it
            
            # status bits
            try:
                tmp[4]                            # check existance
            except IndexError:                    # or raise error
                dic['status_bits'] = ''
            else:                                 # if ok
                dic['status_bits'] = tmp[4]         # set it
            
            # appned bond dictionnary to atom list   
            bonds.append(dic)
        
        return bonds

    def _get_records(self, file):
        """
        Returns records of a mol2 file.
        """
        input_file = open(file)
        all_text = input_file.read()
        paragraphs = all_text.split('@<TRIPOS>')
        records = {}
        for paragraph in paragraphs:
            lines = paragraph.split('\n')
            record_type = lines[0]
            record_data = lines[1:-1]
            records[record_type] = record_data
        return records

